package com.link.base.basic.controller;

import com.link.base.basic.model.ListOfValue;
import com.link.base.basic.service.BasicCommonService;
import com.link.base.cachedt.service.CacheDTService;
import com.link.base.enterprise.model.CoreEnterprise;
import com.link.core.basic.controller.BasicController;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.Filter;
import com.link.core.basic.query.QueryParams;
import com.link.core.basic.query.Sorter;
import com.link.core.basic.service.BasicService;
import com.link.core.basic.service.KeyGenerateService;
import com.link.core.cllog.LCLogger;
import com.link.core.util.JsonUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping({"/link/basic"})
public class PortalBasicController extends BasicController<ListOfValue> {
   private static final Logger logger = LogManager.getLogger(PortalBasicController.class);
   private static final String LOV_REDIS_PREFIX = "LOV_CACHE_";
   private static final int EXPIRE_TIME = 86400;
   public static final String LNK_DATA_UPD_LOV = "LNK_DATA_UPD_LOV";
   @Resource
   private BasicCommonService basicCommonService;
   @Resource
   private CacheDTService cacheDTService;
   @Resource
   private KeyGenerateService keyGenerateService;

   public BasicService<ListOfValue> getBasicService() throws Exception {
      return this.basicCommonService;
   }

   @RequestMapping({"/parentPickList"})
   @ResponseBody
   public Map<String, Object> parentPickList(QueryParams qps, ListOfValue entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap();

      try {
         qps.preDealSecurity(request);
         ListOfValue listOfValue = (ListOfValue)BasicModel.transformClass(ListOfValue.class, qps);
         listOfValue.setActiveFlag("Y");
         List<ListOfValue> list = this.getBasicService().queryByExamplePage(listOfValue);
         result.put("success", true);
         result.put("rows", list);
         result.put("total", getCount(listOfValue, list.size()));
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("PortalBasicController捕获异常: ", e);
         result.put("success", false);
         result.put("rows", new ArrayList());
         result.put("result", e.getMessage());
      }

      return result;
   }

   @PostMapping(
      value = {"/getId"},
      name = "生成记录Id"
   )
   @ResponseBody
   public Map<String, Object> getId(HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         ArrayList<String> arrayList = new ArrayList();
         int ten = 10;

         for(int i = 0; i < ten; ++i) {
            arrayList.add(this.keyGenerateService.keyGenerate());
         }

         result.put("success", true);
         result.put("result", arrayList);
      } catch (Exception var8) {
         Exception e = var8;
         LCLogger.withException(e);
         result.put("success", false);
         result.put("message", e.getMessage());
      }

      return result;
   }

   @RequestMapping({"/flushCache"})
   @ResponseBody
   public Map<String, Object> flushCache(HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap();
      LinkRedisClient jedis = null;

      try {
         jedis = RedisUtil.getJedis();
         String keyPrefix = "LOV_CACHE_";
         Long corpId = UserUtil.getUser() != null ? UserUtil.getUser().getCorpid() : 0L;
         Set<String> set = jedis.keys(keyPrefix + "*_" + corpId);
         set.addAll(jedis.keys(keyPrefix + "*_" + corpId + "_mutex"));
         if (set.size() > 0) {
            String[] keyStrs = new String[set.size()];
            int i = 0;

            for(Iterator<String> it = set.iterator(); it.hasNext(); ++i) {
               String keyStr = (String)it.next();
               keyStrs[i] = keyStr;
            }

            jedis.del(keyStrs);
         }

         result.put("success", true);
      } catch (Exception var16) {
         Exception e = var16;
         logger.error("PortalBasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   @RequestMapping({"/list/{type}"})
   public void list(@PathVariable String type, ListOfValue entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      QueryParams qps = new QueryParams();
      String redisKey = "LOV_CACHE_";
      qps.addFilter(new Filter("type", type.toUpperCase()));
      redisKey = redisKey + type.toUpperCase();
      if (StringUtils.isNotBlank(entity.getParentName())) {
         qps.addFilter(new Filter("parentName", entity.getParentName()));
         redisKey = redisKey + "#1" + entity.getParentName();
      }

      if (StringUtils.isNotBlank(entity.getParentVal())) {
         qps.addFilter(new Filter("parentVal", entity.getParentVal()));
         redisKey = redisKey + "#2" + entity.getParentVal();
      }

      if (null != entity.getParentId()) {
         qps.addFilter(new Filter("parentId", String.valueOf(entity.getParentId())));
         redisKey = redisKey + "#3" + entity.getParentId();
      }

      LinkRedisClient jedis = null;

      Exception e;
      label93: {
         try {
            jedis = RedisUtil.getJedis();
            redisKey = redisKey + "_";
            redisKey = redisKey + (UserUtil.getUser() != null ? UserUtil.getUser().getCorpid() : "");
            String listJson = jedis.get(redisKey);
            if (StringUtils.isBlank(listJson)) {
               break label93;
            }

            jedis.expire(redisKey, 86400);
            RedisUtil.returnResource(jedis);
            responseJson(request, response, listJson);
         } catch (Exception var15) {
            e = var15;
            logger.error(e.toString());
            break label93;
         } finally {
            RedisUtil.returnResource(jedis);
         }

         return;
      }

      qps.addSorter(new Sorter("seq", "ASC"));

      try {
         qps.setPageFlag(false);
         ListOfValue listOfValue = (ListOfValue)BasicModel.transformClass(ListOfValue.class, qps);
         listOfValue.setActiveFlag("Y");
         String json = this.getAndSet(redisKey, listOfValue, 0, (LinkRedisClient)null);
         responseJson(request, response, json);
      } catch (Exception var14) {
         e = var14;
         logger.error("PortalBasicController捕获异常: ", e);
      }

   }

   @RequestMapping({"/listOfVal/types"})
   public void listOfValTypes(String types, ListOfValue entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      LinkRedisClient jedis = null;

      try {
         jedis = RedisUtil.getJedis();
         String startStr = "\"[";
         String endStr = "]\"";
         Long corpId = UserUtil.getUser() != null ? UserUtil.getUser().getCorpid() : 0L;
         String[] typeArray = types.substring(types.indexOf(startStr) + startStr.length(), types.indexOf(endStr)).split(",");
         StringBuilder resultJsonStrb = new StringBuilder("[");

         String jsonStr;
         for(int i = 0; i < typeArray.length; ++i) {
            jsonStr = typeArray[i];
            String redisKey = "LOV_CACHE_" + jsonStr.toUpperCase() + "_" + corpId;
            entity.setActiveFlag("Y");
            entity.setType(jsonStr.toUpperCase());
            entity.setPageFlag(false);
            String jsonStr = this.getAndSet(redisKey, entity, 0, (LinkRedisClient)null);
            if (null != jsonStr && !StringUtils.isBlank(jsonStr) && !"[]".equals(jsonStr)) {
               jsonStr = jsonStr.substring(1, jsonStr.length() - 1);
               resultJsonStrb.append(jsonStr);
               resultJsonStrb.append(",");
               jedis.expire(redisKey, 86400);
            } else {
               logger.error("Can not find lov result with type '" + jsonStr + "' ");
            }
         }

         if (resultJsonStrb != null && resultJsonStrb.length() > 0) {
            String str = resultJsonStrb.substring(0, resultJsonStrb.length() - 1);
            jsonStr = "";
            if (StringUtils.isNull(str)) {
               jsonStr = "{\"success\":true,\"rows\": []}";
            } else {
               jsonStr = "{\"success\":true,\"rows\":" + str + "]}";
            }

            responseJson(request, response, jsonStr);
         } else {
            responseJson(request, response, "{\"success\":true,\"rows\": []}");
         }
      } catch (Exception var19) {
         Exception e = var19;
         logger.error("PortalBasicController捕获异常: ", e);
      } finally {
         RedisUtil.returnResource(jedis);
      }

   }

   private String getAndSet(String key, ListOfValue example, int count, LinkRedisClient jedis) throws Exception {
      logger.debug("getAndSet--> " + key + "; " + count);
      if (jedis == null) {
         jedis = RedisUtil.getJedis();
      }

      String result = null;

      try {
         Exception e;
         try {
            result = jedis.get(key);
            if (result == null) {
               if (example == null) {
                  RedisUtil.returnResource(jedis);
                  e = null;
                  return e;
               }

               String[] stayFields = new String[]{"id", "type", "val", "name", "parentId", "parentType", "parentVal", "parentName", "seq", "defaultValue", "useFlag"};
               String mutexKey = key + "_mutex";
               if (!jedis.exists(mutexKey) && count < 10) {
                  try {
                     String json;
                     try {
                        jedis.setex(mutexKey, 600, "Y");
                        List<ListOfValue> list = this.basicCommonService.queryByExamplePage(example);
                        json = JsonUtil.listToStringIncludeArrays(list, stayFields);
                        jedis.set(key, json);
                        jedis.expire(key, 86400);
                        jedis.del(mutexKey);
                        result = json;
                        this.cacheDTService.hsetCacheDataUpdTime("LNK_DATA_UPD_LOV", key, jedis, (String)null);
                     } catch (Exception var20) {
                        Exception e = var20;
                        logger.error("PortalBasicController捕获异常: ", e);
                        json = null;
                        return json;
                     }
                  } finally {
                     jedis.del(mutexKey);
                     RedisUtil.returnResource(jedis);
                  }
               } else {
                  jedis.expire(mutexKey, 10);
                  RedisUtil.returnResource(jedis);
                  if (count <= 10) {
                     Thread.sleep(100L);
                     result = this.getAndSet(key, example, count + 1, jedis);
                  }
               }
            }
         } catch (Exception var22) {
            e = var22;
            logger.error(e.toString());
         }
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   public void beforeImport(ListOfValue entity, HttpServletRequest request) throws Exception {
      QueryParams queryParams = new QueryParams();
      queryParams.addFilter(new Filter("name", entity.getCorpName()));
      CoreEnterprise enterprise = (CoreEnterprise)BasicModel.transformClass(CoreEnterprise.class, queryParams);
      enterprise.setPageFlag(false);
   }
}
